@@ -18,6 +18,14 @@ fn main() -> Result<(), anyhow::Error> {
         )
         .subcommand(Command::new("build").about("Build the site"))
         .subcommand(Command::new("serve").about("Serve the site"))
+        .subcommand(
+            Command::new("new").about("Create a new post page").arg(
+                Arg::new("name")
+                    .value_name("NAME")
+                    .required(true)
+                    .help("The name of the post page"),
+            ),
+        )
         .get_matches();
 
     match matches.subcommand() {
@@ -37,6 +45,17 @@ fn main() -> Result<(), anyhow::Error> {
             SiteServer::new("127.0.0.1:8080").run()?;
             Ok(())
         }
+        Some(("new", arg_matches)) => {
+            let mut name = arg_matches.value_of("name").unwrap().to_string();
+            if !name.ends_with(".md") {
+                name.push_str(".md");
+            }
+            let mut site = SiteBuilder::new();
+
+            site.create_page(&name)?;
+
+            Ok(())
+        }
         _ => unreachable!(),
     }
 }
